<!---
The contents of this file are subject to the Mozilla Public License Version 1.1
(the "License"); you may not use this file except in compliance with the
License. You may obtain a copy of the License at http://www.mozilla.org/MPL/
 
Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
the specific language governing rights and limitations under the License.
 
The Original Code is comprised of the ADF directory
 
The Initial Developer of the Original Code is
PaperThin, Inc.  Copyright (c) 2009-2018.
All Rights Reserved.
 
By downloading, modifying, distributing, using and/or accessing any files
in this directory, you agree to the terms and conditions of the applicable
end user license agreement.
--->

<!---
/* *************************************************************** */
Author: 	   				
	PaperThin Inc.
Name:
	device_size_options_render.cfc
Summary:
	Renders up to 6 property fields, including:
		xxs,xs,sm,md,lg
	
	extends="ADF.extensions.customfields.adf-form-field-renderer-base"
Version:
	1.0
History:
	2017-12-06 - GAC - Updated Label spacing
--->
<cfcomponent displayName="Device Size Options Render" extends="ADF.extensions.customfields.adf-form-field-renderer-base">

<cfset gProps = 'xxs,xs,sm,md,lg'>

<cffunction name="renderControl" returntype="void" access="public">
	<cfargument name="fieldName" type="string" required="yes">
	<cfargument name="fieldDomID" type="string" required="yes">
	<cfargument name="value" type="string" required="yes">
	
	<cfscript>
		var inputParameters = application.ADF.data.duplicateStruct(arguments.parameters);
		var currentValue = arguments.value;	// the field's current value
		var inputParameters = setDefaultParameters(argumentCollection=arguments);
		var currValueStruct = structNew();
		var i = 0;
		var prop = '';
		
		for( i=1; i lte ListLen(gProps); i=i+1 )
		{
			prop = ListGetAt(gProps,i);
			if( NOT StructKeyExists(inputParameters, '#prop#Value') )
				inputParameters['#prop#Value'] = '';
			if( NOT StructKeyExists(inputParameters, '#prop#Display') )	
				inputParameters['#prop#Display'] = '';
			if( NOT StructKeyExists(inputParameters, '#prop#Default') )
				inputParameters['#prop#Default'] = '';
		}		
		
		if( len(currentValue) AND isJSON(currentValue) )
		{
			currValueStruct = deserializeJSON(currentValue);

			for( i=1; i lte ListLen(gProps); i=i+1 )
			{
				prop = ListGetAt(gProps,i);
	
				// set the current value into the props Default setting
				if( structKeyExists(currValueStruct, prop) )
					inputParameters['#prop#Default'] = currValueStruct[prop];
			}
		}
	</cfscript>
	
	<cfif Not StructKeyExists(request,'cft_device_size_options')>
		<cfoutput>
			<style>
				/** this max-width causes scroll bars on first tabs if this field is on a second tab 
					##cs_commondlg .CS_FormFieldControlContainer { max-width: 660px; } **/
				.dsoFlexcontainer {
					display: flex;
					flex-wrap: wrap;
					flex-direction: row;
					margin-left: -5px;
				}
				.dsoFlexitem {
					margin-bottom: 5px;
				}
				.dsoFlexitem > span {
					padding-left: 5px;
				}
			</style>
		</cfoutput>
		<cfset request.cft_device_size_options = 1>
	</cfif>
	
	<!---
		Render the field using a flex box container to wrap on each row
	--->	
	<cftry>
	
		<cfoutput><div class="dsoFlexcontainer"></cfoutput>
			
			<cfscript>
				// loop over each property
				for( i=1; i lte ListLen(gProps); i=i+1 )
				{
					prop = ListGetAt(gProps, i);
					RenderProperty(inputParameters, prop, currValueStruct);
				}
			</cfscript>
	
			<cfoutput>
				<!--- // hidden field to store the value --->
				<input type="hidden" name="#arguments.fieldName#" value='#currentValue#' id="#inputParameters.fieldID#" class="#inputParameters.fieldClass#">
			</div>
			</cfoutput>
		
		<cfscript>
			renderJSFunctions(argumentCollection=arguments,fieldParameters=inputParameters);
		</cfscript>
		
	<cfcatch>
		<cfdump var="#cfcatch#" expand="no" label="ERROR">
	</cfcatch>
	</cftry>
	
</cffunction>

<cffunction name="RenderProperty" access="private" output="Yes" returntype="void">
	<cfargument name="params" required="Yes" type="struct">
	<cfargument name="prop" required="Yes" type="string">
	<cfargument name="currentValuesStruct" required="Yes" type="struct">
	
	<cfscript>
		var inputParameters = arguments.params;
		var possibleValuesList = inputParameters['#arguments.prop#Value'];
		var possibleDisplayList = '';
		var defaultValue = inputParameters['#arguments.prop#Default'];			
		var elData = '';
		var renderAsList = 0;
		var i = 0;
		var vList = '';
		var dList = '';
		var item = '';

		var value = defaultValue;
		
		// Check if Property is to be displayed
		if( structKeyExists(inputParameters, 'display#arguments.prop#') AND 
				inputParameters['display#arguments.prop#'] eq 1 )
		{
			if( Len(possibleValuesList) eq 0 )
			{
				// Empty list - assume user can enter data manually so display input control
				renderAsList = 0;
			}
			
			// Check if List comes from Custom Element data
			else if( FindNoCase('element=', possibleValuesList) )
			{
				renderAsList = 1;
				
				try 
				{
					// Lookup values
					elData = getElementValues( possibleValuesList );

					if( len(elData.error) )
						possibleValuesList = elData.error;
					else
					{
						possibleValuesList = elData.valuelist;
						possibleDisplayList = elData.displaylist;
					}
				}
				catch(any e)
				{
					WriteOutput('<p>There is an error in your properties configuration for #arguments.prop#. Please review it and try it again. The error message was:<br>#e.message#</p>');
				}
			}	
			
			// Assume list is a comma delimited list, NOT empty and not pointer Custom Element
			else if( ListLen(possibleValuesList) gt 0 ) 
			{
				renderAsList = 1;
				 
				// check if the possible values list is in the form: displayvalue=value
				if( FindNoCase( '=', possibleValuesList) )
				{
					for( i=1; i lte ListLen(possibleValuesList); i = i+ 1 )
					{
						item = ListGetAt(possibleValuesList, i);
						dList = ListAppend( dList, ListFirst(item, '=') );
						vList = ListAppend( vList, ListLast(item, '=') );
					}
					
					// reset lists
					possibleValuesList = vList;
					possibleDisplayList = dList;
				}
				else	// normal comma delimited list
				{
					possibleDisplayList = '';
				}
				
			}
			else	// Treat as empty list
			{
				renderAsList = 0;
			}
			
			// Render Selection List
			if( renderAsList )
			{
				RenderSelectionList(
					show = inputParameters['display#arguments.prop#'],
					label = inputParameters['#arguments.prop#Label'] & ":",
					FieldID = inputParameters.FieldID,
					name = arguments.prop,
					value = value,
					possibleDisplays = possibleDisplayList,
					possibleValues = possibleValuesList);
			}
			else
			{
					// value in text field
					WriteOutput('<div class="dsoFlexitem">');
					WriteOutput('<span>');
					WriteOutput( inputParameters['#arguments.prop#Label'] & ":" );
					WriteOutput('</span>');
					WriteOutput('<input type="text" name="#arguments.prop#" value="#value#" size="20">');
					WriteOutput('</div>');
			}
		}
		else
		{
			// Render default value in hidden field
			WriteOutput('<input type="hidden" name="#arguments.prop#" value="#defaultValue#">');
		}	
	</cfscript>	
	
</cffunction>

<cffunction name="renderJSFunctions" returntype="void" access="private">
	<cfargument name="fieldName" type="string" required="yes">
	<cfargument name="fieldDomID" type="string" required="yes">
	<cfargument name="value" type="string" required="yes">
	<cfargument name="fieldParameters" type="struct" required="yes">
	
	<cfscript>
		var inputParameters = application.ADF.data.duplicateStruct(arguments.fieldParameters);
		var cftFooterJS = '';
	</cfscript>

	<cfif NOT StructKeyExists(request,'renderDeviceSettingOptionJS')>
		<cfset request.renderDeviceSettingOptionJS = 1>

		<cfsavecontent variable="cftFooterJS">
			<cfoutput>
			<script type="text/javascript">
			function onChangeDeviceSizeOptionProp(fieldID) 
			{
				<cfloop index="prop" list="#gProps#">
					if( jQuery('##' + fieldID + '_#prop#').length ) 
						var curr#prop# = jQuery('##' + fieldID + '_#prop#').val();
					else 
						var curr#prop# = '';
				</cfloop>
				
				var currObj = { "xxs":currxxs, "xs":currxs, "sm":currsm, "md":currmd, "lg":currlg };
				var currJSON = JSON.stringify(currObj);				

				// Set value into field				
				jQuery('##' + fieldID ).val(currJSON);
			}
			</script>
			</cfoutput>
		</cfsavecontent>		
		
	</cfif>	

	<cfscript>
		application.ADF.scripts.addFooterJS(cftFooterJS,"TERTIARY");
	</cfscript>

	<cfsavecontent variable="cftFooterJS">
		<cfoutput>
			<script type="text/javascript">
				onChangeDeviceSizeOptionProp('#inputParameters.fieldID#');
			</script>
		</cfoutput>
	</cfsavecontent>

	<cfscript>
		application.ADF.scripts.addFooterJS(cftFooterJS,"TERTIARY");
	</cfscript>
</cffunction>

<cffunction name="setDefaultParameters" returntype="struct" access="private">
	<cfargument name="fieldName" type="string" required="yes">
	<cfargument name="fieldDomID" type="string" required="yes">
	<cfargument name="value" type="string" required="yes">
	
	<cfscript>
		var inputParameters = application.ADF.data.duplicateStruct(arguments.parameters);
		
		// Validate if the property field has been defined
		if ( NOT StructKeyExists(inputParameters, "fldID") OR LEN(inputParameters.fldID) LTE 0 )
			inputParameters.fldID = arguments.fieldName;
		if ( NOT StructKeyExists(inputParameters, "fieldID") )
			inputParameters.fieldID = arguments.fieldName;
		if ( (NOT StructKeyExists(inputParameters, "fieldClass")) OR ( LEN(TRIM(inputParameters.fieldClass)) LTE 0) )
			inputParameters.fieldClass = "";
		
		if ( NOT StructKeyExists(inputParameters, "displayxxs") )
			inputParameters.displayxxs = 0;
		if ( NOT StructKeyExists(inputParameters, "displayxs") )
			inputParameters.displayxs = 0;
		if ( NOT StructKeyExists(inputParameters, "displaysm") )
			inputParameters.displaysm = 0;
		if ( NOT StructKeyExists(inputParameters, "displaymd") )
			inputParameters.displaymd = 0;
		if ( NOT StructKeyExists(inputParameters, "displaylg") )
			inputParameters.displaylg = 0;
		
		if ( NOT StructKeyExists(inputParameters, "xxsLabel") OR LEN(TRIM(inputParameters.xxsLabel)) eq 0 )
			inputParameters.xxsLabel = "Small Phone";
		if ( NOT StructKeyExists(inputParameters, "xsLabel") OR LEN(TRIM(inputParameters.xsLabel)) eq 0 )
			inputParameters.xsLabel = "Phone";
		if ( NOT StructKeyExists(inputParameters, "smLabel") OR LEN(TRIM(inputParameters.smLabel)) eq 0 )
			inputParameters.smLabel = "Tablet";
		if ( NOT StructKeyExists(inputParameters, "mdLabel") OR LEN(TRIM(inputParameters.mdLabel)) eq 0 )
			inputParameters.mdLabel = "Desktop";
		if ( NOT StructKeyExists(inputParameters, "lgLabel") OR LEN(TRIM(inputParameters.lgLabel)) eq 0 )
			inputParameters.lgLabel = "Large Desktop";
				
		if ( NOT StructKeyExists(inputParameters, "xxsDefault") )
			inputParameters.xxsDefault = "";
		if ( NOT StructKeyExists(inputParameters, "xsDefault") )
			inputParameters.xsDefault = "";
		if ( NOT StructKeyExists(inputParameters, "smDefault") )
			inputParameters.smDefault = "";
		if ( NOT StructKeyExists(inputParameters, "mdDefault") )
			inputParameters.mdDefault = "";
		if ( NOT StructKeyExists(inputParameters, "lgDefault") )
			inputParameters.lgDefault = "";
		
		// Remove any labels from the possibleValues String
		if ( NOT StructKeyExists(inputParameters, "possibleValues") )
			inputParameters.possibleValues = "";
		
		return inputParameters;
	</cfscript>
</cffunction>

<cffunction name="getElementValues" returntype="struct" access="private">
	<cfargument name="elementConfig" type="string" required="yes">

	<!--- example: element=Photo Size,displayfield=title,valuefield=sizeid  --->
	<cfscript>
		var currValueList = "";
		var currDisplayList = "";
		var currListStruct = structNew();
		
		var assocArray = StructNew();
		var v = '';
		var d = '';
		var i = 0;
		
		var checkElement = '';
		var currElement = '';
		var currElDisplay = '';
		var currElValue = '';
		var currElementData = '';
		
		var v = '';
		var d = '';
		var key = '';
	</cfscript>

	<cfloop list="#arguments.elementConfig#" item="listitem">
		<cfscript>
			if( listitem contains "element=" )
				currElement = listlast(listitem,"=");
			else if( listitem contains "displayfield=" )
				currElDisplay = listlast(listitem,"=");
			else if( listitem contains "valuefield=" )
				currElValue = listlast(listitem,"=");
		</cfscript>
	</cfloop>

	<cfset checkElement = application.adf.cedata.elementExists(currElement)>

	<cfif checkElement>
		<cfscript>
			currElementData = application.adf.cedata.getCEData(currElement);
			
			for( i=1; i lte ArrayLen(currElementData); i=i+1 )
			{
				v = currElementData[i].values[currElValue];
				d = currElementData[i].values[currElDisplay];
				if( d eq '' )
					d = v;
				assocArray[v] = d;
			}
			sortedKeys = StructSort( assocArray, 'textnocase', 'asc' );

			for( i=1; i lte ArrayLen(sortedKeys); i=i+1 )
			{
				key = sortedKeys[i];
				currValueList = ListAppend(currValueList, key);
				currDisplayList = ListAppend(currDisplayList, assocArray[key]);				
			}

			currListStruct.valuelist = currValueList;
			currListStruct.displaylist = currDisplayList;
			currListStruct.error = "";
		</cfscript>
		
	<cfelse>
		<cfset currListStruct.error = "Element '#currElement#' does not exist. Please check your properties configuration and try again.">
	</cfif>

	<cfreturn currListStruct>

</cffunction>


<cfscript>
	/*
		IMPORTANT: Since loadResourceDependencies() is using ADF.scripts loadResources methods, getResourceDependencies() and
		loadResourceDependencies() must stay in sync by accounting for all of required resources for this Custom Field Type.
	*/
	public void function loadResourceDependencies()
	{
		// Load registered Resources via the ADF scripts_2_0
		application.ADF.scripts.loadJQuery(noConflict=true);
	}

	public string function getResourceDependencies()
	{
		return "jQuery";
	}
	
	public numeric function getMinWidth()
	{
		return 900;
	}
</cfscript>


<!---
/* *************************************************************** */
Author:
	PaperThin, Inc.
Name:
	$RenderSelectionList
Summary:
	Renders a selection list of possible field values.
Returns:
	Void
Arguments:
	Boolean - Show
	String - Label
	String - FieldID
	String - Name
	String - Value
	String - possibleValues
	String - possibleDisplays
	String - style
History:
	2017-11-10 - Created
--->

<cffunction name="RenderSelectionList" access="private" output="Yes" returntype="void" hint="renders a selection list of possible values.">
	<cfargument name="Show" type="Boolean" required="Yes">
	<cfargument name="Label" type="String" required="Yes">
	<cfargument name="FieldID" type="string" required="Yes">
	<cfargument name="name" type="string" required="Yes">
	<cfargument name="value" type="string" required="Yes">
	<cfargument name="possibleValues" type="string" required="Yes">
	<cfargument name="possibleDisplays" type="string" required="No" default="">
	<cfargument name="style" type="string" required="no" default="">
	
	<cfscript>
		var index="";
		var currListValue = "";
		var currListDisplay = "";
		var itm = 0;
		var c = '';
		var StyleAttr = '';

		if( arguments.possibleDisplays eq '' )
			arguments.possibleDisplays = arguments.possibleValues;
			
		if ( LEN(TRIM(arguments.style)) ) 
			StyleAttr = 'style="#arguments.style#"';
	</cfscript>

	<cfif arguments.show>
		<cfoutput>
			<div class="dsoFlexitem" id="#arguments.FieldID#_#arguments.Name#_wrapper">
				<span>#arguments.label#</span>
				<select id="#arguments.FieldID#_#arguments.Name#" name="#arguments.FieldID#_#arguments.Name#" #StyleAttr# onchange="onChangeDeviceSizeOptionProp('#FieldID#');">
					</cfoutput>
					
					<cfloop from="1" to="#listlen(arguments.possibleValues)#" index="itm">
					
						<cfscript>
							currListValue = listGetAt(arguments.possibleValues,itm);
							currListDisplay = listGetAt(arguments.possibleDisplays,itm);
						</cfscript>

						<cfoutput><option value="#currListValue#" <cfif TRIM(arguments.value) eq TRIM(currListValue)> selected="selected"</cfif>>#TRIM(currListDisplay)#</option></cfoutput>

					</cfloop>

				<cfoutput>
				</select>
			</div>
		</cfoutput>
	<cfelse>
		<cfoutput><input type="hidden" id="#arguments.FieldID#_#arguments.Name#" name="#arguments.FieldID#_#arguments.Name#" value="#TRIM(arguments.value)#"></cfoutput>
	</cfif>
</cffunction>

<!---
/* *************************************************************** */
Author:
	PaperThin, Inc.
Name:
	$RenderInputField
Summary:
	Renders an generic input field
Returns:
	Void
Arguments:
	Boolean - Show
	String - Label
	String - FieldID
	String - Name
	String - Value
History:
	2014-09-15 - Created
--->
<cffunction name="RenderTextInput" access="private" output="Yes" returntype="void" hint="renders a selection list of possible values.">
	<cfargument name="Show" type="Boolean" required="Yes">
	<cfargument name="Label" type="String" required="Yes">
	<cfargument name="FieldID" type="string" required="Yes">
	<cfargument name="name" type="string" required="Yes">
	<cfargument name="value" type="string" required="Yes">

	<cfif arguments.show>
		<cfoutput>
			<span>#arguments.label#</span>
			<input type="text" id="#arguments.FieldID#_#arguments.Name#" name="#arguments.FieldID#_#arguments.Name#" value="#TRIM(arguments.value)#" size="4" onKeyUp="onChange_#FieldID#();" style="text-align:right;">
		</cfoutput>
	<cfelse>
		<cfoutput><input type="hidden" id="#arguments.FieldID#_#arguments.Name#" name="#arguments.FieldID#_#arguments.Name#" value="#TRIM(arguments.value)#"></cfoutput>
	</cfif>
</cffunction>

<!---
/* *************************************************************** */
Author: 	
	PaperThin, Inc. 
Name:
	$RenderNumberInput
Summary:
	Renders an generic input field 
Returns:
	Void
Arguments:
	Boolean - Show
	String - Label
	String - FieldID
	String - Name
	String - Value
History:
	2017-11-13 - Created
	2017-12-06 - GAC - Updated Label spacing
--->
<cffunction name="RenderNumberInput" access="private" output="Yes" returntype="void" hint="renders a selection list of possible values.">
	<cfargument name="Show" type="Boolean" required="Yes">
	<cfargument name="Label" type="String" required="Yes">
	<cfargument name="FieldID" type="string" required="Yes">
	<cfargument name="name" type="string" required="Yes">
	<cfargument name="value" type="string" required="Yes">
	
	<cfif arguments.show>
		<cfoutput>
			<span>#arguments.label#</span>
			<input type="number" id="#arguments.FieldID#_#arguments.Name#" name="#arguments.FieldID#_#arguments.Name#" value="#TRIM(arguments.value)#" size="3" min="0" max="999" onchange="onChange_#FieldID#();" style="width:40px;text-align:right;margin-right:3px;">px&nbsp;&nbsp;
		</cfoutput>	
	<cfelse>
		<cfoutput><input type="hidden" id="#arguments.FieldID#_#arguments.Name#" name="#arguments.FieldID#_#arguments.Name#" value="#TRIM(arguments.value)#"></cfoutput>
	</cfif>
</cffunction>

</cfcomponent>
