<!---
The contents of this file are subject to the Mozilla Public License Version 1.1
(the "License"); you may not use this file except in compliance with the
License. You may obtain a copy of the License at http://www.mozilla.org/MPL/

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
the specific language governing rights and limitations under the License.

The Original Code is comprised of the ADF directory

The Initial Developer of the Original Code is
PaperThin, Inc.  Copyright (c) 2009-2018.
All Rights Reserved.

By downloading, modifying, distributing, using and/or accessing any files
in this directory, you agree to the terms and conditions of the applicable
end user license agreement.
--->

<!---
/* *************************************************************** */
Author:
	PaperThin Inc.
Name:
	padding_props.cfm
Summary:
	Renders 4 input controls to capture padding values.
Version:
	1.0.0
History:
	2014-09-15 - Created
	2014-09-29 - GAC - Added Padding Value normalization to remove the label from the default values
	2015-05-12 - DJM - Updated the field version to 2.0
	2015-09-02 - DRM - Add getResourceDependencies support, bump version
	2016-02-16 - GAC - Added getResourceDependencies and loadResourceDependencies support to the Render
						  - Added the getResources check to the Props
						  - Bumped field version
	2017-11-07 - GAC - Moved resources to addFooterJS cfsavecontent block
	2017-11-29 - GAC - Added the renderNumberInput function to render input fields as type='number'
	2017-11-30 - GAC - Fixed issue with checked options not displaying properly when props is re-opened.
	2017-12-08 - DRM - Added units dropdown
						  - Bumped field version
	2017-12-12 - JTP - Added units title text to selection list
--->
<cfsetting enablecfoutputonly="Yes" showdebugoutput="No">

<!--- // if this module loads resources, do it here.. --->
<!---<cfscript>
    // No resources to load
</cfscript>--->

<!--- ... then exit if all we're doing is detecting required resources --->
<cfif Request.RenderState.RenderMode EQ "getResources">
  <cfexit>
</cfif>

<cfscript>
	prefix = attributes.prefix;
	currentValues = attributes.currentValues;
	typeid = attributes.typeid;
	formname = attributes.formname;

	// Variable for the version of the field - Display in Props UI.
	fieldVersion = "2.1.5";

	// initialize some of the attributes variables
	optionsList = "top,right,bottom,left";
	showOptionsList = "showTop,showRight,showBottom,showLeft";


	// Possible Values List
	PossibleValues = '10,9,8,7,6,5,4,3,2,1,0';

	if( StructKeyExists(currentValues, "PossibleValues") )
		PossibleValues = application.ADF.paddingSettings.normalizePaddingValues(PaddingValues=currentValues.PossibleValues);

	// Loop over Options to set defaults and check for current saved values
	for ( i=1; i LTE ListLen(optionsList); i=i+1 ) {
		optionItem = listGetAt(optionsList,i);

		variables[optionItem] = 0;
		if( StructKeyExists(currentValues,optionItem) )
			variables[optionItem] = application.ADF.paddingSettings.normalizePaddingValues(PaddingValues=currentValues[optionItem]);
	}

	// Loop over ShowOptions to set defaults and check for current saved values
	for ( s=1; s LTE ListLen(showOptionsList); s=s+1 ) {
		showOptionItem = listGetAt(showOptionsList,s);

		variables[showOptionItem] = 1;
		if( !StructKeyExists(currentValues,showOptionItem) OR currentValues[showOptionItem] EQ 0 )
			variables[showOptionItem] = 0;
	}

	unitsOptions = listToArray('px,%,em,rem,vw,vh,vmin,vmax,none');
	units = structKeyExists(currentValues, 'units') ? currentValues.units : 'px';
	
	unitsHints = ArrayNew(1);
	unitsHintsList = "";
		
	unitsHints[1] = 'Spacing in pixels';
	unitsHints[2] = 'Spacing relative to the percentage of the parent';		
	unitsHints[3] = 'Spacing relative to the font-size of the element (2em = 2 times the size of the current font)';
	unitsHints[4] = 'Spacing relative to font-size of the root element';
	unitsHints[5] = 'Spacing relative to 1% of the width of the viewport';
	unitsHints[6] = 'Spacing relative to 1% of the height of the viewport';		
	unitsHints[7] = "Spacing relative to 1% of viewport's smaller dimension";		
	unitsHints[8] = "Spacing relative to 1% of viewport's larger dimension";		
	unitsHints[9] = "Don't set spacing";	
	cnt = 1;
	attrs = '';
</cfscript>


<cfsavecontent variable="cftFooterJS"><cfoutput><script type="text/javascript">
<!--
	fieldProperties['#typeid#'].paramFields = "#prefix#PossibleValues,#prefix#Units,#prefix#Top,#prefix#Right,#prefix#Bottom,#prefix#Left,#prefix#ShowTop,#prefix#ShowRight,#prefix#ShowBottom,#prefix#ShowLeft";

	function unitsOnChange(units)
	{
		var elems, i;
		if (units === 'none')
		{
			elems = document.querySelectorAll('.#prefix##prefix#hasUnits input');
			for (i = 0; i < elems.length; i++)
				elems[i].value = '';
		}
		else
		{
			elems = document.querySelectorAll('.#prefix#unitsDisplay');
			for (i = 0; i < elems.length; i++)
				elems[i].innerHTML = units;
		}
		var disp = (units === 'none') ? 'none' : '';
		elems = document.querySelectorAll('.#prefix#hasUnits');
		for (i = 0; i < elems.length; i++)
			elems[i].style.display = disp;
	}

	unitsOnChange('#units#');
//-->
</script></cfoutput></cfsavecontent>

<cfscript>
	application.ADF.scripts.addFooterJS(cftFooterJS,"TERTIARY");
</cfscript>

<cfoutput>
	<table>
		<tr>
			<td class="cs_dlgLabelBold" valign="top" nowrap="nowrap">Possible Values:</td>
			<td class="cs_dlgLabelSmall">
				<input type="text" name="#prefix#PossibleValues" id="#prefix#PossibleValues" class="cs_dlgControl" value="#PossibleValues#" size="60">
				<br/>If Possible Values are specified, the user is presented with a selection list of values.
				<br/>Otherwise, an input control will be presented.
				<br/><br/>
			</td>
		</tr>

		<tr>
			<td class="cs_dlgLabelBold" valign="top" nowrap="nowrap">Show:</td>
			<td class="cs_dlgLabelSmall">
				<input type="checkbox" name="#prefix#ShowTop" id="#prefix#ShowTop" value="1" class="cs_dlgControl" <cfif ShowTop eq 1>checked="checked"</cfif>><label for="#prefix#ShowTop">Top</label> &nbsp;
				<input type="checkbox" name="#prefix#ShowRight" id="#prefix#ShowRight" value="1" class="cs_dlgControl" <cfif ShowRight eq 1>checked="checked"</cfif>><label for="#prefix#ShowRight">Right</label> &nbsp;
				<input type="checkbox" name="#prefix#ShowBottom" id="#prefix#ShowBottom" value="1" class="cs_dlgControl" <cfif ShowBottom eq 1>checked="checked"</cfif>><label for="#prefix#ShowBottom">Bottom</label> &nbsp;
				<input type="checkbox" name="#prefix#ShowLeft" id="#prefix#ShowLeft" value="1" class="cs_dlgControl" <cfif ShowLeft eq 1>checked="checked"</cfif>><label for="#prefix#ShowLeft">Left</label> &nbsp;
			</td>
		</tr>

		<tr>
			<td class="cs_dlgLabelSmall" colspan="2">&nbsp;</td>
		</tr>

		<tr>
			<td class="cs_dlgLabelBold" valign="top" nowrap="nowrap">Units Default:</td>
			<td class="cs_dlgLabelSmall">
				<select name="#prefix#Units" id="#prefix#Units" class="cs_dlgControl" onchange="unitsOnChange(this.value)">
					</cfoutput>
					
					<cfset cnt = 1>
					<cfloop array="#unitsOptions#" index="option">
						<cfscript>
							attrs = 'title="#unitsHints[cnt]#"';
							cnt = cnt + 1;	
							
							if( option eq units )
								attrs = attrs & ' selected="selected"';
						</cfscript>	
						
						<cfoutput><option value="#option#" #attrs#>#option#</option></cfoutput>
					</cfloop>
					
					<cfoutput>
				</select>
			</td>
		</tr>
		<tr class="#prefix#hasUnits">
			<td class="cs_dlgLabelBold" valign="top" nowrap="nowrap">Top Default:</td>
			<td class="cs_dlgLabelSmall">
				<input type="text" name="#prefix#Top" id="#prefix#Top" class="cs_dlgControl" value="#top#" size="4" style="text-align:right;" onchange="valueOnChange()"><span class="#prefix#unitsDisplay">#units#</span>
			</td>
		</tr>
		<tr class="#prefix#hasUnits">
			<td class="cs_dlgLabelBold" valign="top" nowrap="nowrap">Right Default:</td>
			<td class="cs_dlgLabelSmall">
				<input type="text" name="#prefix#Right" id="#prefix#Right" class="cs_dlgControl" value="#right#" size="4" style="text-align:right;" onchange="valueOnChange()"><span class="#prefix#unitsDisplay">#units#</span>
			</td>
		</tr>
		<tr class="#prefix#hasUnits">
			<td class="cs_dlgLabelBold" valign="top" nowrap="nowrap">Bottom Default:</td>
			<td class="cs_dlgLabelSmall">
				<input type="text" name="#prefix#Bottom" id="#prefix#Bottom" class="cs_dlgControl" value="#bottom#" size="4" style="text-align:right;" onchange="valueOnChange()"><span class="#prefix#unitsDisplay">#units#</span>
			</td>
		</tr>
		<tr class="#prefix#hasUnits">
			<td class="cs_dlgLabelBold" valign="top" nowrap="nowrap">Left Default:</td>
			<td class="cs_dlgLabelSmall">
				<input type="text" name="#prefix#Left" id="#prefix#Left" class="cs_dlgControl" value="#left#" size="4" style="text-align:right;" onchange="valueOnChange()"><span class="#prefix#unitsDisplay">#units#</span>
			</td>
		</tr>

		<tr>
			<td class="cs_dlgLabelSmall" colspan="2" style="font-size:7pt;">
				<hr />
				ADF Custom Field v#fieldVersion#
			</td>
		</tr>
	</table>
</cfoutput>