<!---
The contents of this file are subject to the Mozilla Public License Version 1.1
(the "License"); you may not use this file except in compliance with the
License. You may obtain a copy of the License at http://www.mozilla.org/MPL/

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
the specific language governing rights and limitations under the License.

The Original Code is comprised of the ADF directory

The Initial Developer of the Original Code is
PaperThin, Inc.  Copyright (c) 2009-2018.
All Rights Reserved.

By downloading, modifying, distributing, using and/or accessing any files
in this directory, you agree to the terms and conditions of the applicable
end user license agreement.
--->

<!---
/* *************************************************************** */
Author:
	PaperThin Inc.
Name:
	padding_settings_render.cfc
Summary:
	Custom field to build the Date/Time records.
	This field generates a collection of Date and Times for the field.
Version:
	1.0.0
History:
	2014-09-15 - Created
	2014-09-29 - GAC - Added Padding Value normalization to remove the label from the default values
	2015-04-28 - DJM - Added own CSS
	2015-09-11 - GAC - Replaced duplicate() with Server.CommonSpot.UDF.util.duplicateBean()
	2016-02-09 - GAC - Updated duplicateBean() to use data_2_0.duplicateStruct()
	2016-02-16 - GAC - Added getResourceDependencies support
	                 - Added loadResourceDependencies support
	                 - Moved resource loading to the loadResourceDependencies() method
   2017-11-07 - GAC - Moved resources to addFooterJS cfsavecontent block
   					  - Enabled jQuery noConflict mode
   2017-11-29 - GAC - Added the renderNumberInput function to render input fields as type='number'
	2017-12-06 - GAC - Updated Label spacing
	2017-12-08 - DRM - Added units support
	2017-12-12 - JTP - Added units title text to selection list
--->
<cfcomponent displayName="PaddingSettings Render" extends="ADF.extensions.customfields.adf-form-field-renderer-base">

<cffunction name="renderControl" returntype="void" access="public">
	<cfargument name="fieldName" type="string" required="yes">
	<cfargument name="fieldDomID" type="string" required="yes">
	<cfargument name="value" type="string" required="yes">

	<cfscript>
		var inputParameters = application.ADF.data.duplicateStruct(arguments.parameters);
		var currentValue = arguments.value;	// the field's current value
		var units = '';
		var top = '';
		var right = '';
		var bottom = '';
		var left = '';
		var unitsOptions = 'px,%,em,rem,vw,vh,vmin,vmax,none';		
		var unitsHints = ArrayNew(1);
		var unitsHintsList = '';
		
		unitsHints[1] = 'Spacing in pixels';
		unitsHints[2] = 'Spacing relative to the percentage of the parent';		
		unitsHints[3] = 'Spacing relative to the font-size of the element (2em = 2 times the size of the current font)';
		unitsHints[4] = 'Spacing relative to font-size of the root element';
		unitsHints[5] = 'Spacing relative to 1% of the width of the viewport';
		unitsHints[6] = 'Spacing relative to 1% of the height of the viewport';		
		unitsHints[7] = "Spacing relative to 1% of viewport's smaller dimension";		
		unitsHints[8] = "Spacing relative to 1% of viewport's larger dimension";		
		unitsHints[9] = "Don't set spacing";	
		
		unitsHintsList = ArrayToList(unitsHints);		

		inputParameters = setDefaultParameters(argumentCollection=arguments);

		if( currentValue eq '' )
		{
			top = application.ADF.paddingSettings.normalizePaddingValues(PaddingValues=inputParameters.top);
			right = application.ADF.paddingSettings.normalizePaddingValues(PaddingValues=inputParameters.right);
			bottom = application.ADF.paddingSettings.normalizePaddingValues(PaddingValues=inputParameters.bottom);
			left = application.ADF.paddingSettings.normalizePaddingValues(PaddingValues=inputParameters.left);
			units = inputParameters.units;
		}
		else
		{
			top = application.ADF.paddingSettings.normalizePaddingValues(PaddingValues=ListGetAt( currentValue, 1, ' ' ) );
			right = application.ADF.paddingSettings.normalizePaddingValues(PaddingValues=ListGetAt( currentValue, 2, ' ' ) );
			bottom = application.ADF.paddingSettings.normalizePaddingValues(PaddingValues=ListGetAt( currentValue, 3, ' ' ) );
			left = application.ADF.paddingSettings.normalizePaddingValues(PaddingValues=ListGetAt( currentValue, 4, ' ' ) );
			units = reReplace(listFirst(currentValue, " "), "[0-9]", "", "ALL");
			if (units == '')
				units = 'none';
		}

		currentValue = top & '#units# ' & right & '#units# ' & bottom & '#units# ' & left & '#units#'; // overwritten by js, which takes into account whether field is hidden, by reading form field value, hidden ones are always 0

		// TODO: make sure each value in the possible values string is numeric
	</cfscript>

	<cfoutput>
		<div>
			<!--- renderSelectionList(show, Label, FieldID, name, value, possibleValues, units, containerClass) --->
			#application.ADF.paddingSettings.renderSelectionList(true,'Units:',inputParameters.FieldID,'Units',units,unitsOptions,'',unitsHintsList)#
			<cfif LEN(TRIM(inputParameters.possibleValues))>
				#application.ADF.paddingSettings.renderSelectionList(inputParameters.showTop,'Top:',inputParameters.FieldID,'Top',top,inputParameters.possibleValues)#
				#application.ADF.paddingSettings.renderSelectionList(inputParameters.showRight,'Right:',inputParameters.FieldID,'Right',right,inputParameters.possibleValues)#
				#application.ADF.paddingSettings.renderSelectionList(inputParameters.showBottom,'Bottom:',inputParameters.FieldID,'Bottom',bottom,inputParameters.possibleValues)#
				#application.ADF.paddingSettings.renderSelectionList(inputParameters.showLeft,'Left:',inputParameters.FieldID,'Left',left,inputParameters.possibleValues)#
			<cfelse>
				#application.ADF.paddingSettings.renderNumberInput(inputParameters.showTop,'Top:',inputParameters.FieldID,'Top',top)#
				#application.ADF.paddingSettings.renderNumberInput(inputParameters.showRight,'Right:',inputParameters.FieldID,'Right',right)#
				#application.ADF.paddingSettings.renderNumberInput(inputParameters.showBottom,'Bottom:',inputParameters.FieldID,'Bottom',bottom)#
				#application.ADF.paddingSettings.renderNumberInput(inputParameters.showLeft,'Left:',inputParameters.FieldID,'Left',left)#
			</cfif>
			<!--- // hidden field to store the value --->
			<input type="hidden" name="#arguments.fieldName#" value="#currentValue#" id="#inputParameters.fieldID#" class="#inputParameters.fieldClass#">
		</div>
	</cfoutput>
	<cfscript>
		renderJSFunctions(argumentCollection=arguments,fieldParameters=inputParameters);
	</cfscript>
</cffunction>

<cffunction name="renderJSFunctions" returntype="void" access="private">
	<cfargument name="fieldName" type="string" required="yes">
	<cfargument name="fieldDomID" type="string" required="yes">
	<cfargument name="value" type="string" required="yes">
	<cfargument name="fieldParameters" type="struct" required="yes">

	<cfscript>
		var inputParameters = application.ADF.data.duplicateStruct(arguments.fieldParameters);
		var cftFooterJS = '';
	</cfscript>

<cfsavecontent variable="cftFooterJS"><cfoutput><script type="text/javascript">
<!--
function onChange_#inputParameters.fieldID#()
{
	var u = jQuery('###inputParameters.fieldID#_Units').val();
	var t, r, b, l, value;
	if (u !== 'none')
	{
		t = jQuery('###inputParameters.fieldID#_Top').val() || 0;
		r = jQuery('###inputParameters.fieldID#_Right').val() || 0;
		b = jQuery('###inputParameters.fieldID#_Bottom').val() || 0;
		l = jQuery('###inputParameters.fieldID#_Left').val() || 0;
		value = t + u + ' ' + r + u + ' ' + b + u + ' ' + l + u;
		jQuery('.#inputParameters.fieldID#_fieldContainer').show();
	}
	else
	{
		jQuery('.#inputParameters.fieldID#_fieldContainer').hide();
		jQuery('###inputParameters.fieldID#_Top,###inputParameters.fieldID#_Right,###inputParameters.fieldID#_Bottom,###inputParameters.fieldID#_Left').val('');
		value = '';
	}
	jQuery('###inputParameters.fieldID#').val(value);
}
onChange_#inputParameters.fieldID#();
//-->
</script></cfoutput></cfsavecontent>

	<cfscript>
		application.ADF.scripts.addFooterJS(cftFooterJS,"TERTIARY");
	</cfscript>
</cffunction>

<cffunction name="setDefaultParameters" returntype="struct" access="private">
	<cfargument name="fieldName" type="string" required="yes">
	<cfargument name="fieldDomID" type="string" required="yes">
	<cfargument name="value" type="string" required="yes">
	<cfscript>
		var inputParameters = application.ADF.data.duplicateStruct(arguments.parameters);

		// Validate if the property field has been defined
		if ( NOT StructKeyExists(inputParameters, "fldID") OR LEN(inputParameters.fldID) LTE 0 )
			inputParameters.fldID = arguments.fieldName;
		if ( NOT StructKeyExists(inputParameters, "fieldID") )
			inputParameters.fieldID = arguments.fieldName;
		if ( (NOT StructKeyExists(inputParameters, "fieldClass")) OR ( LEN(TRIM(inputParameters.fieldClass)) LTE 0) )
			inputParameters.fieldClass = "";

		// Remove any labels from the possibleValues String
		if ( StructKeyExists(inputParameters, "possibleValues") AND LEN(TRIM(inputParameters.possibleValues)) )
			inputParameters.possibleValues = application.ADF.paddingSettings.normalizePaddingValues(PaddingValues=inputParameters.possibleValues);
		else
			inputParameters.possibleValues = "";

		if( NOT StructKeyExists( inputParameters, 'ShowTop' ) )
			inputParameters.ShowTop = 0;
		if( NOT StructKeyExists( inputParameters, 'ShowRight' ) )
			inputParameters.ShowRight = 0;
		if( NOT StructKeyExists( inputParameters, 'ShowBottom' ) )
			inputParameters.ShowBottom = 0;
		if( NOT StructKeyExists( inputParameters, 'ShowLeft' ) )
			inputParameters.ShowLeft = 0;

		if( NOT StructKeyExists( inputParameters, 'Units' ) )
			inputParameters.Units = 'px';

		return inputParameters;
	</cfscript>
</cffunction>


<cfscript>
	/*
		IMPORTANT: Since loadResourceDependencies() is using ADF.scripts loadResources methods, getResourceDependencies() and
		loadResourceDependencies() must stay in sync by accounting for all of required resources for this Custom Field Type.
	*/
	public void function loadResourceDependencies()
	{
		// Load registered Resources via the ADF scripts_2_0
		application.ADF.scripts.loadJQuery(noConflict=true);
	}
	public string function getResourceDependencies()
	{
		return "jQuery";
	}
	public numeric function getMinWidth()
	{
		return 800;
	}
</cfscript>

</cfcomponent>